@@ -12,7 +12,7 @@
  *
  * Also finds out IP of specified MAC
  *
- * $Id: arping.c,v 1.24 2000/09/11 21:39:16 marvin Exp $
+ * $Id: arping.c,v 1.27 2000/09/18 19:12:17 marvin Exp $
  */
 /*
  *  Copyright (C) 2000 Marvin (marvin@nss.nu)
@@ -54,13 +54,18 @@
 #include "openbsd.h"
 #endif
 
+#if SOLARIS
+#include "solaris.h"
+#include "netinet/arp.h"
+#endif
+
 #if 0
 #define DEBUG(a) a
 #else
 #define DEBUG(a)
 #endif
 
-const float version = 0.91;
+const float version = 0.92;
 
 struct ether_addr *mymac;
 u_char eth_xmas[ETH_ALEN] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
@@ -96,7 +101,7 @@ void sigint(int i);
 
 void usage(int ret)
 {
-	printf("arping %1.1f [ -v ] [ -r ] [ -d ] [-0 ] [ -c count ] "
+	printf("arping %1.2f [ -v ] [ -r ] [ -d ] [ -0 ] [ -c count ] "
 	       "[ -i <interface> ] <host/ip/MAC>\n", version);
 	exit(ret);
 }
@@ -145,6 +150,9 @@ void alasend(int i)
 		exit(1);
 	}
 	alarm(1);
+#if SOLARIS
+	signal(SIGALRM, alasend);
+#endif
 }
 
 void sigint(int i)
@@ -334,11 +342,11 @@ int main(int argc, char **argv)
 	}
 
 	if (!(linkint = libnet_open_link_interface(ifname, ebuf))) {
-		fprintf(stderr, "libnet_get_hwaddr(): %s\n", ebuf);
+		fprintf(stderr, "libnet_open_link_interface(): %s\n", ebuf);
 		exit(1);
 	}
 	
-	if (!(mymac = libnet_get_hwaddr(NULL, (u_char*)ifname,  ebuf))) {
+	if (!(mymac = libnet_get_hwaddr(linkint, (u_char*)ifname,  ebuf))) {
 		fprintf(stderr, "libnet_get_hwaddr(): %s\n", ebuf);
 		exit(1);
 	}
@@ -350,11 +358,23 @@ int main(int argc, char **argv)
 		fprintf(stderr, "libnet_get_ipaddr(): %s\n", ebuf);
 		exit(1);
 	}
-	
-	if (-1 == libnet_init_packet(LIBNET_ETH_H + LIBNET_IP_H
-				     + LIBNET_ICMP_H, &packet)) {
-		fprintf(stderr, "libnet_init_packet(): error\n");
-		exit(1);
+
+	if (searchmac) {
+		if (-1 == libnet_init_packet(LIBNET_ETH_H + LIBNET_IP_H
+					     + LIBNET_ICMP_ECHO_H, &packet)) {
+			fprintf(stderr, "libnet_init_packet(): error\n");
+			exit(1);
+		}
+	} else {
+		/*
+		 * this makes it work on solaris.
+		 * not sure if LIBNET_ICMP_H is needed though, but it works
+		 */
+		if (-1 == libnet_init_packet(LIBNET_ETH_H + LIBNET_ARP_H
+					     + LIBNET_ICMP_H, &packet)) {
+			fprintf(stderr, "libnet_init_packet(): error\n");
+			exit(1);
+		}
 	}
 	
 	if (verbose) {
@@ -420,7 +440,7 @@ int main(int argc, char **argv)
 					   NULL,
 					   0,
 					   packet + LIBNET_ETH_H)) {
-			fprintf(stderr, "libnet_init_packet(): error\n");
+			fprintf(stderr, "libnet_build_arp(): error\n");
 			exit(1);
 		}
 		
